@@ -1,26 +1,32 @@
 import json
 import logging
+from typing import Any, Dict, List, Optional, Union, cast
+
+from requests import Response
 
-from urbanairship.devices import ChannelTags
 from urbanairship import common
-from urbanairship.push.payload import device_types
+from urbanairship.client import BaseClient
+from urbanairship.devices.tag import ChannelTags
 
 logger = logging.getLogger("urbanairship")
 
 
 class NamedUser(object):
-    """Perform various operations on a named user object"""
+    """
+    Perform various operations on a named user object
 
-    def __init__(self, airship, named_user_id=None):
+    :param airship: Required. An instance of urbanairship.Airship.
+    """
 
+    def __init__(self, airship: BaseClient, named_user_id: Optional[str] = None) -> None:
         self._airship = airship
         self.named_user_id = named_user_id
-        self.channel_id = None
-        self.email_address = None
-        self.device_type = None
+        self.channel_id: Optional[str] = None
+        self.email_address: Optional[str] = None
+        self.device_type: Optional[str] = None
 
     @property
-    def _channel_associate_payload(self):
+    def _channel_associate_payload(self) -> Dict:
         """
         creates the paylaod for channel_id associate and disassociate calls
         """
@@ -32,7 +38,7 @@ class NamedUser(object):
         return payload
 
     @property
-    def _email_associate_payload(self):
+    def _email_associate_payload(self) -> Dict:
         """
         creates the payload for email_address associate and disassociate calls
         """
@@ -41,7 +47,7 @@ class NamedUser(object):
             "email_address": self.email_address,
         }
 
-    def _dis_associate(self, url, body):
+    def _dis_associate(self, url: str, body: Dict) -> Response:
         response = self._airship.request(
             method="POST",
             body=json.dumps(body),
@@ -52,7 +58,7 @@ class NamedUser(object):
 
         return response
 
-    def associate(self, channel_id, device_type=None):
+    def associate(self, channel_id: str, device_type: Optional[str] = None) -> Response:
         """Associate a channel_id  with a named user ID.
         Either channel_id and device_type OR email_address must be included.
 
@@ -76,9 +82,9 @@ class NamedUser(object):
             body=self._channel_associate_payload,
         )
 
-    def email_associate(self, email_address):
+    def email_associate(self, email_address: str) -> Response:
         """Associate an email_address with a named user id. This call is for a literal
-            email address.
+        email address.
 
         :param email_address: Required. The email address to associate.
 
@@ -94,7 +100,7 @@ class NamedUser(object):
             body=self._email_associate_payload,
         )
 
-    def disassociate(self, channel_id, device_type=None):
+    def disassociate(self, channel_id: str, device_type: Optional[str] = None) -> Response:
         """Disassociate a channel with a named user ID
 
         :param channel_id: The ID of the channel you would like to disassociate
@@ -116,7 +122,7 @@ class NamedUser(object):
             body=self._channel_associate_payload,
         )
 
-    def email_disassociate(self, email_address):
+    def email_disassociate(self, email_address: str) -> Response:
         """Disassociate an email_address with a named user ID
 
         :param channel_id: The email_address you would like to disassociate
@@ -133,7 +139,7 @@ class NamedUser(object):
             body=self._email_associate_payload,
         )
 
-    def lookup(self):
+    def lookup(self) -> Union[Dict, str]:
         """Lookup a single named user
 
         :return: The named user payload for the named user ID
@@ -146,17 +152,24 @@ class NamedUser(object):
             version=3,
             params={"id": self.named_user_id},
         )
-        return response.json()
+        return cast(Union[Dict, str], response.json())
+
+    def tag(
+        self,
+        group: str,
+        add: Optional[List] = None,
+        remove: Optional[List] = None,
+        set: Optional[List] = None,
+    ) -> Dict:
+        """Add, remove, or set tags on a named user.
 
-    def tag(self, group, add=None, remove=None, set=None):
-        """Add, remove, or set tags on a named user
         :param add: A list of tags to add
         :param remove: A list of tags to remove
         :param set: A list of tags to set
         :param group: The Tag group for the add, remove, and set operations
         """
         if self.named_user_id:
-            payload = {"audience": {"named_user_id": self.named_user_id}}
+            payload: Dict[str, Any] = {"audience": {"named_user_id": self.named_user_id}}
         else:
             raise ValueError("A named user ID is required for modifying tags")
 
@@ -190,22 +203,29 @@ class NamedUser(object):
             version=3,
         )
 
-        return response.json()
+        return cast(Dict[Any, Any], response.json())
 
-    def update(self, associate=None, disassociate=None, tags=None, attributes=None):
+    def update(
+        self,
+        associate: Optional[List] = None,
+        disassociate: Optional[List] = None,
+        tags: Optional[Dict] = None,
+        attributes: Optional[List] = None,
+    ) -> Response:
         """
         Create or update a named user by associating/disassociating channels
-            and adding/removing tags and attributes in a single request.
+        and adding/removing tags and attributes in a single request.
+
         If a channel has an assigned named user and you make an additional call to
-            associate that same channel with a new named user, the original named user
-            association will be removed and the new named user and associated data will
-            take its place. Additionally, all tags associated to the original named
-            user cannot be used to address this channel unless they are also associated
-            with the new named user.
+        associate that same channel with a new named user, the original named user
+        association will be removed and the new named user and associated data will
+        take its place. Additionally, all tags associated to the original named
+        user cannot be used to address this channel unless they are also associated
+        with the new named user.
 
         Please see https://docs.airship.com/api/ua/#operation-api-named_users-named_user_id-post
 
-        :param assocaite: Optional. List of objects to associate with the named user
+        :param associate: Optional. List of objects to associate with the named user
         :param disassociate: Optional. List of objects to disassociate from the named
             user
         :param tags: Optional. Dictionary of set, add, remove objects to apply to
@@ -219,7 +239,7 @@ class NamedUser(object):
                 "At least one of associate, disassociate, tags, or attributes must be included"
             )
 
-        body = {}
+        body: Dict[str, Any] = {}
 
         if associate:
             body["associate"] = associate
@@ -233,14 +253,14 @@ class NamedUser(object):
         response = self._airship.request(
             method="POST",
             body=json.dumps(body),
-            url=self._airship.urls.get("named_user_url") + self.named_user_id,
+            url=f'{self._airship.urls.get("named_user_url")}{self.named_user_id}',
             content_type="application/json",
             version=3,
         )
 
         return response
 
-    def attributes(self, attributes):
+    def attributes(self, attributes: Optional[List]) -> Response:
         """
         Set or remove attributes on a named user.
         A single request body may contain a set or remove field, or both, or a single
@@ -249,8 +269,9 @@ class NamedUser(object):
         If an attribute request is partially valid, i.e. at least one attribute exists,
         Airship returns a 200 with a warning in containing a list of attributes that
         failed to update.
-        Please see https://docs.airship.com/api/ua/#operation-api-named_users-named_user_id-attributes-post
-            for more about using Airship attributes.
+        Please see
+        https://docs.airship.com/api/ua/#operation-api-named_users-named_user_id-attributes-post
+        for more about using Airship attributes.
 
         :params attributes: Required. A list of attribute objects to set or remove on
             the named user.
@@ -261,9 +282,7 @@ class NamedUser(object):
         response = self._airship.request(
             method="POST",
             body=json.dumps({"attributes": attributes}),
-            url=self._airship.urls.get("named_user_url")
-            + self.named_user_id
-            + "attributes",
+            url=f'{self._airship.urls.get("named_user_url")}{self.named_user_id}{attributes}',
             content_type="application/json",
             version=3,
         )
@@ -271,7 +290,7 @@ class NamedUser(object):
         return response
 
     @classmethod
-    def uninstall(cls, airship, named_users):
+    def uninstall(cls, airship: BaseClient, named_users: List[str]) -> Response:
         """
         Disassociate and delete all channels associated with the named_user_id(s) and
         also delete the named_user_id(s). This call removes all channels associated
@@ -300,11 +319,11 @@ class NamedUser(object):
         return response
 
     @classmethod
-    def from_payload(cls, payload):
+    def from_payload(cls, payload: Dict):
         """
         Create NamedUser object based on results from a NamedUserList iterator.
-        :param payload: Payload used to create the NamedUser object
 
+        :param payload: Payload used to create the NamedUser object
         """
         for key in payload:
             setattr(cls, key, payload[key])
@@ -315,10 +334,10 @@ class NamedUser(object):
 class NamedUserList(common.IteratorParent):
     """Retrieves a list of NamedUsers"""
 
-    next_url = None
-    data_attribute = "named_users"
+    next_url: Optional[str] = None
+    data_attribute: str = "named_users"
 
-    def __init__(self, airship):
+    def __init__(self, airship: BaseClient):
         self.next_url = airship.urls.get("named_user_url")
         super(NamedUserList, self).__init__(airship, None)
 
@@ -326,9 +345,20 @@ class NamedUserList(common.IteratorParent):
 class NamedUserTags(ChannelTags):
     """Modify the tags for named users"""
 
-    def __init__(self, airship):
+    def __init__(self, airship: BaseClient) -> None:
         super(NamedUserTags, self).__init__(airship)
         self.url = airship.urls.get("named_user_tag_url")
 
-    def set_audience(self, user_ids):
+    def set_audience(
+        self,
+        user_ids: Optional[List[str]] = None,
+        ios: Optional[str] = None,
+        android: Optional[str] = None,
+        amazon: Optional[str] = None,
+        web: Optional[str] = None,
+    ) -> None:
+        """Set the named user ids to be modified.
+
+        :param user_ids: A list of named user ids.
+        """
         self.audience["named_user_id"] = user_ids
