@@ -1,42 +1,50 @@
-import json
-import gzip
 import collections
 import datetime
+import gzip
+import json
+from io import TextIOWrapper
+from typing import Any, Dict, Optional, cast
+
+from requests import Response
 
 from urbanairship import common
+from urbanairship.client import BaseClient
 
 CHUNK = 16 * 1024
 
 
-class StaticList(object):
-    def __init__(self, airship, name):
+class StaticList:
+    def __init__(self, airship: BaseClient, name: str) -> None:
         self.airship = airship
         self.name = name
         self.description = None
         self.extra = None
 
-    def create(self):
+    def create(self) -> Dict[Any, Any]:
+        """Create a Static List"""
         payload = {"name": self.name}
-        if self.description is not None:
+        if self.description:
             payload["description"] = self.description
-        if self.extra is not None:
+        if self.extra:
             payload["extra"] = self.extra
 
-        body = json.dumps(payload).encode("utf-8")
+        body = json.dumps(payload)
         response = self.airship._request(
-            "POST",
-            body,
-            self.airship.urls.get("lists_url"),
-            "application/json",
+            method="POST",
+            body=body,
+            url=self.airship.urls.get("lists_url"),
+            content_type="application/json",
             version=3,
         )
-        return response.json()
+        result = response.json()
+        return cast(Dict[Any, Any], result)
 
-    def upload(self, csv_file):
+    def upload(self, csv_file: TextIOWrapper) -> Dict[Any, Any]:
         """Upload a CSV file to a static list
 
         :param csv_file: open file descriptor with two column format:
             identifier_type, identifier
+
         :return: http response
         """
 
@@ -50,10 +58,11 @@ class StaticList(object):
             version=3,
             encoding="gzip",
         )
-        return response.json()
+        return cast(Dict[Any, Any], response.json())
 
-    def update(self):
+    def update(self) -> Dict[Any, Any]:
         """Update the metadata in a static list
+
         :return: http response
         """
 
@@ -70,17 +79,15 @@ class StaticList(object):
         body = json.dumps(payload).encode("utf-8")
         url = self.airship.urls.get("lists_url") + self.name
 
-        response = self.airship._request(
-            "PUT", body, url, "application/json", version=3
-        )
-
-        return response.json()
+        response = self.airship._request("PUT", body, url, "application/json", version=3)
+        result = response.json()
+        return cast(Dict[Any, Any], result)
 
     @classmethod
-    def download(cls, airship, list_name):
+    def download(cls, airship: BaseClient, list_name: str) -> Response:
         """
         Allows you to download the contents of a static list. Alias and named_user
-            values are resolved to channels.
+        values are resolved to channels.
 
         :param airship: Required. An urbanairship.Airship instance.
         :param list_name: Required. Name of an existing list to download.
@@ -96,19 +103,18 @@ class StaticList(object):
         return response
 
     @classmethod
-    def from_payload(cls, payload, airship):
-        obj = cls(airship, payload["name"])
+    def from_payload(cls, payload: Dict[Any, Any], airship: BaseClient):
         for key in payload:
-            if key in "created" or key in "last_updated":
-                payload[key] = datetime.datetime.strptime(
-                    payload[key], "%Y-%m-%dT%H:%M:%S"
-                )
-            setattr(obj, key, payload[key])
-        return obj
+            if key == "created" or key == "last_updated":
+                payload[key] = datetime.datetime.strptime(payload[key], "%Y-%m-%dT%H:%M:%S")
+            setattr(cls, key, payload[key])
+        return cls
 
     def lookup(self):
         """
-        :return: Information about the static list
+        Get Information about the static list
+
+        :return: urbanairship.StaticList objects
         """
 
         url = self.airship.urls.get("lists_url") + self.name
@@ -116,19 +122,20 @@ class StaticList(object):
         payload = response.json()
         return self.from_payload(payload, self.airship)
 
-    def delete(self):
+    def delete(self) -> Response:
         """
-        :return: Delete the static list
+        Delete the static list
         """
         url = self.airship.urls.get("lists_url") + self.name
         return self.airship._request("DELETE", None, url, version=3)
 
 
 class StaticLists(common.IteratorParent):
-    next_url = None
-    data_attribute = "lists"
+    next_url: Optional[str] = None
+    data_attribute: str = "lists"
 
     def __init__(self, airship):
+        """Gets an iterable listing of existing static lists"""
         self.next_url = airship.urls.get("lists_url")
         super(StaticLists, self).__init__(airship, None)
 
@@ -187,7 +194,7 @@ class GzipCompressReadStream(object):
         return self
 
     def __next__(self):
-        if self.is_finished is True:
+        if self.is_finished:
             raise StopIteration
         else:
             return self.read(CHUNK)
