@@ -1,30 +1,60 @@
 import json
 import logging
-import warnings
+from typing import Any, Dict, List, Optional, Union
+
+from requests import Response
 
 from urbanairship import devices
-
+from urbanairship.client import BaseClient
 
 logger = logging.getLogger("urbanairship")
 
 
+class PushResponse(object):
+    """Response to a successful push notification send or schedule.
+
+    Right now this is a fairly simple wrapper around the json payload response,
+    but making it an object gives us some flexibility to add functionality
+    later.
+
+    """
+
+    ok: Optional[bool] = None
+    localized_ids: Optional[List] = None
+    push_ids: Optional[List] = None
+    schedule_url: Optional[str] = None
+    operation_id: Optional[str] = None
+    payload: Optional[Dict] = None
+
+    def __init__(self, response: Response):
+        data = response.json()
+        self.localized_ids = data.get("localized_ids", [])
+        self.push_ids = data.get("push_ids")
+        self.schedule_url = data.get("schedule_urls", [])
+        self.operation_id = data.get("operation_id")
+        self.ok = data.get("ok")
+        self.payload = data
+
+    def __str__(self) -> str:
+        return "Response Payload: {0}".format(self.payload)
+
+
 class Push(object):
     """A push notification. Set audience, message, etc, and send."""
 
-    def __init__(self, airship):
+    def __init__(self, airship: BaseClient) -> None:
         self._airship = airship
-        self.audience = None
-        self.notification = None
-        self._device_types = None
-        self.options = None
-        self.campaigns = None
-        self.message = None
-        self.in_app = None
-        self.localizations = None
+        self.audience: Optional[Union[Dict, List[Dict]]] = None
+        self.notification: Optional[Dict[str, Any]] = None
+        self.options: Optional[Dict[str, Any]] = None
+        self.campaigns: Optional[Dict[str, Any]] = None
+        self.message: Optional[Dict[str, Any]] = None
+        self.in_app: Optional[Dict[str, Any]] = None
+        self.localizations: Optional[Dict[str, Any]] = None
 
     @property
-    def payload(self):
-        data = {
+    def payload(self) -> Dict[str, Any]:
+        data: Dict[str, Any] = {
             "audience": self.audience,
             "notification": self.notification,
             "device_types": self.device_types,
@@ -42,18 +72,14 @@ class Push(object):
         return data
 
     @property
-    def device_types(self):
+    def device_types(self) -> List:
         return self._device_types
 
     @device_types.setter
-    def device_types(self, types):
-        if types == "all" or (len(types) == 1 and types[0] == "all"):
-            warnings.warn(
-                "The device type 'all' has been deprecated.", DeprecationWarning
-            )
+    def device_types(self, types: List) -> None:
         self._device_types = types
 
-    def validate(self):
+    def validate(self) -> PushResponse:
         """
         Test push payload against the validate endpoint. No sends will result from this
         method being called. This method is otherwise identical to the `send` method.
@@ -69,7 +95,7 @@ class Push(object):
 
         return PushResponse(response)
 
-    def send(self):
+    def send(self) -> PushResponse:
         """Send the notification.
 
         :returns: :py:class:`PushResponse` object with ``push_ids`` and
@@ -77,23 +103,15 @@ class Push(object):
         :raises AirshipFailure: Request failed.
         :raises Unauthorized: Authentication failed.
         :raises ValueError: Required keys missing or incorrect values included.
+        :raises ConnectionFailure: Connection failed.
         """
         if "email" in self.payload["notification"]:
             if self.payload["device_types"] == "all":
-                raise ValueError(
-                    "device_types cannot be all when including an email override"
-                )
+                raise ValueError("device_types cannot be all when including an email override")
             if "email" not in self.payload["device_types"]:
-                raise ValueError(
-                    "email must be in device_types if email override is included"
-                )
-        if (
-            "email" in self.payload["device_types"]
-            and "email" not in self.payload["notification"]
-        ):
-            raise ValueError(
-                "email override must be included when email is in device_types"
-            )
+                raise ValueError("email must be in device_types if email override is included")
+        if "email" in self.payload["device_types"] and "email" not in self.payload["notification"]:
+            raise ValueError("email override must be included when email is in device_types")
 
         body = json.dumps(self.payload)
         response = self._airship._request(
@@ -105,14 +123,12 @@ class Push(object):
         )
 
         data = response.json()
-        logger.info(
-            "Push successful. push_ids: %s", ", ".join(data.get("push_ids", []))
-        )
+        logger.info("Push successful. push_ids: %s", ", ".join(data.get("push_ids", [])))
 
         return PushResponse(response)
 
     @classmethod
-    def message_center_delete(cls, airship, push_id):
+    def message_center_delete(cls, airship: BaseClient, push_id: str) -> Response:
         """
         Delete a Message Center message completely, removing it from every user's inbox.
         This is an asynchronous call; a success response means that the deletion has
@@ -143,16 +159,16 @@ class Push(object):
 class ScheduledPush(object):
     """A scheduled push notification. Set schedule, push, and send."""
 
-    def __init__(self, airship):
+    def __init__(self, airship: BaseClient):
         self._airship = airship
-        self.schedule = None
-        self.recurring = None
-        self.name = None
-        self.push = None
-        self.url = None
+        self.schedule: Optional[Dict[str, Any]] = None
+        self.recurring: Optional[Dict[str, Any]] = None
+        self.name: Optional[str] = None
+        self.push: Optional[Union[Push, "CreateAndSendPush"]] = None
+        self.url: Optional[str] = None
 
     @classmethod
-    def from_url(cls, airship, url):
+    def from_url(cls, airship: BaseClient, url: str):
         """Load an existing scheduled push from its URL."""
 
         sched = cls(airship)
@@ -172,24 +188,24 @@ class ScheduledPush(object):
         return sched
 
     @classmethod
-    def from_payload(cls, payload, id_key, airship):
+    def from_payload(cls, payload: Dict, id_key: str, airship: BaseClient):
         """Create based on results from a ScheduledList iterator."""
         obj = cls(airship)
-        obj._schedule_id = payload[id_key]
         for key in payload:
             setattr(obj, key, payload[key])
         return obj
 
     @property
-    def payload(self):
-        if hasattr(self.push, "merge_data"):  # create template payload
-            data = self.push.payload
+    def payload(self) -> Dict:
+        if not self.push:
+            raise AttributeError("Must set push attribute to build payload.")
+        if hasattr(self.push, "merge_data"):
+            # create template payload
+            data: Dict = self.push.payload
             data["schedule"] = self.schedule
         elif isinstance(self.push, CreateAndSendPush):  # create create and send payload
-            if "scheduled_time" not in self.schedule:
-                raise ValueError(
-                    "only scheduled_time supported with create and send schedules"
-                )
+            if self.schedule is None or "scheduled_time" not in self.schedule:
+                raise ValueError("only scheduled_time supported with create and send schedules")
             data = {"schedule": self.schedule, "push": self.push.payload}
         else:
             data = {"schedule": self.schedule, "push": self.push.payload}
@@ -203,7 +219,7 @@ class ScheduledPush(object):
         return data
 
     @property
-    def api_url(self):
+    def api_url(self) -> str:
         if hasattr(self.push, "merge_data"):
             url = self._airship.urls.get("schedule_template_url")
         elif isinstance(self.push, CreateAndSendPush):
@@ -213,13 +229,14 @@ class ScheduledPush(object):
 
         return url
 
-    def send(self):
+    def send(self) -> PushResponse:
         """Schedule the notification
 
         :returns: :py:class:`PushResponse` object with ``schedule_url`` and
             other response data.
         :raises AirshipFailure: Request failed.
         :raises Unauthorized: Authentication failed.
+        :raises ConnectionFailure: Connection failed.
 
         """
         response = self._airship._request(
@@ -244,7 +261,7 @@ class ScheduledPush(object):
 
         return PushResponse(response)
 
-    def validate(self):
+    def validate(self) -> PushResponse:
         """Validates a scheduled push for sending"""
         response = self._airship._request(
             method="POST",
@@ -254,9 +271,9 @@ class ScheduledPush(object):
             version=3,
         )
 
-        return response
+        return PushResponse(response)
 
-    def pause(self):
+    def pause(self) -> Response:
         """Pause a recurring schedule"""
         if not self.url:
             raise ValueError("Cannot pause ScheduledPush without url.")
@@ -267,7 +284,7 @@ class ScheduledPush(object):
 
         return response
 
-    def resume(self):
+    def resume(self) -> Response:
         """Resume a paused recurring schedule"""
         if not self.url:
             raise ValueError("Cannot resume ScheduledPush without url.")
@@ -278,18 +295,16 @@ class ScheduledPush(object):
 
         return response
 
-    def cancel(self):
+    def cancel(self) -> Response:
         """Cancel a previously scheduled notification."""
         if not self.url:
             raise ValueError("Cannot cancel ScheduledPush without url.")
 
-        response = self._airship._request(
-            method="DELETE", body=None, url=self.url, version=3
-        )
+        response = self._airship._request(method="DELETE", body=None, url=self.url, version=3)
 
         return response
 
-    def update(self):
+    def update(self) -> PushResponse:
         if not self.url:
             raise ValueError("Cannot update ScheduledPush without url.")
         body = json.dumps(self.payload)
@@ -313,15 +328,15 @@ class ScheduledPush(object):
 class TemplatePush(object):
     """A personalized push notification. Set details and send."""
 
-    def __init__(self, airship):
-        self._airship = airship
-        self.audience = None
-        self.device_types = None
-        self.merge_data = None
+    def __init__(self, airship: BaseClient) -> None:
+        self._airship: BaseClient = airship
+        self.audience: Optional[Dict] = None
+        self.device_types: Optional[List] = None
+        self.merge_data: Optional[Dict] = None
 
     @property
-    def payload(self):
-        data = {
+    def payload(self) -> Dict:
+        data: Dict[str, Any] = {
             "audience": self.audience,
             "device_types": self.device_types,
             "merge_data": self.merge_data,
@@ -329,13 +344,14 @@ class TemplatePush(object):
 
         return data
 
-    def send(self):
+    def send(self) -> PushResponse:
         """Send the personalized notification.
 
         :returns: :py:class:`PushResponse` object with ``push_ids`` and
             other response data.
         :raises AirshipFailure: Request failed.
         :raises Unauthorized: Authentication failed.
+        :raises ConnectionFailure: Connection failed.
 
         """
 
@@ -355,9 +371,7 @@ class TemplatePush(object):
         )
 
         data = response.json()
-        logger.info(
-            "Push successful. push_ids: %s", ", ".join(data.get("push_ids", []))
-        )
+        logger.info("Push successful. push_ids: %s", ", ".join(data.get("push_ids", [])))
 
         return PushResponse(response)
 
@@ -376,18 +390,18 @@ class CreateAndSendPush(object):
         CreateAndSend.device_types.
     """
 
-    def __init__(self, airship, channels=[]):
+    def __init__(self, airship: BaseClient, channels: List):
         self._airship = airship
         self.channels = channels
-        self.notification = None
-        self.campaigns = None
+        self.notification: Optional[Dict] = None
+        self.campaigns: Optional[Dict] = None
 
     @property
-    def device_types(self):
+    def device_types(self) -> List[str]:
         return self._device_types
 
     @device_types.setter
-    def device_types(self, values):
+    def device_types(self, values: List[str]):
         accepted_device_types = ("sms", "email", "open::")
 
         if len(values) != 1:
@@ -424,8 +438,8 @@ class CreateAndSendPush(object):
         self._channels = value
 
     @property
-    def payload(self):
-        data = {
+    def payload(self) -> Dict[str, Any]:
+        data: Dict[str, Any] = {
             "audience": self.audience,
             "notification": self.notification,
             "device_types": self.device_types,
@@ -434,41 +448,37 @@ class CreateAndSendPush(object):
             data["campaigns"] = self.campaigns
         return data
 
-    def _email_audience(self):
-        addresses = []
+    def _email_audience(self) -> Dict[str, Any]:
+        addresses: List = []
         for email in self.channels:
             if not isinstance(email, devices.Email):
-                raise TypeError(
-                    "Can only use email channels when device_types is email"
-                )
+                raise TypeError("Can only use email channels when device_types is email")
             addresses.append(email.create_and_send_audience)
-        audience = {"create_and_send": addresses}
+        audience: Dict[str, Any] = {"create_and_send": addresses}
 
         return audience
 
-    def _sms_audience(self):
-        addresses = []
+    def _sms_audience(self) -> Dict[str, Any]:
+        addresses: List = []
         for sms in self.channels:
             if not isinstance(sms, devices.Sms):
                 raise TypeError("Can only use Sms objects when device_types is sms")
             addresses.append(sms.create_and_send_audience)
-        audience = {"create_and_send": addresses}
+        audience: Dict[str, Any] = {"create_and_send": addresses}
 
         return audience
 
-    def _open_channel_audience(self):
-        addresses = []
+    def _open_channel_audience(self) -> Dict[str, Any]:
+        addresses: List = []
         for open_channel in self.channels:
             if not isinstance(open_channel, devices.OpenChannel):
-                raise TypeError(
-                    "Can only use OpenChannel objects when device_types is open::"
-                )
+                raise TypeError("Can only use OpenChannel objects when device_types is open::")
             addresses.append(open_channel.create_and_send_audience)
-        audience = {"create_and_send": addresses}
+        audience: Dict[str, Any] = {"create_and_send": addresses}
 
         return audience
 
-    def send(self):
+    def send(self) -> PushResponse:
         """Send the notification.
 
         :returns: :py:class:`PushResponse` object with ``push_ids`` and
@@ -476,6 +486,8 @@ class CreateAndSendPush(object):
         :raises AirshipFailure: Request failed.
         :raises Unauthorized: Authentication failed.
         :raises ValueError: Required keys missing or incorrect values included.
+        :raises ConnectionFailure: Connection failed.
+
         """
         body = json.dumps(self.payload)
         response = self._airship._request(
@@ -489,32 +501,3 @@ class CreateAndSendPush(object):
         logger.info("Create and Send successful")
 
         return PushResponse(response)
-
-
-class PushResponse(object):
-    """Response to a successful push notification send or schedule.
-
-    Right now this is a fairly simple wrapper around the json payload response,
-    but making it an object gives us some flexibility to add functionality
-    later.
-
-    """
-
-    ok = None
-    localized_ids = None
-    push_ids = None
-    schedule_url = None
-    operation_id = None
-    payload = None
-
-    def __init__(self, response):
-        data = response.json()
-        self.localized_ids = data.get("localized_ids", [])
-        self.push_ids = data.get("push_ids")
-        self.schedule_url = data.get("schedule_urls", [])
-        self.operation_id = data.get("operation_id")
-        self.ok = data.get("ok")
-        self.payload = data
-
-    def __str__(self):
-        return "Response Payload: {0}".format(self.payload)
